------------------------------------------------------------------------------
----                                                                      ----
----  RS-232 baudrate generator                                           ----
----                                                                      ----
----  http://www.opencores.org/                                           ----
----                                                                      ----
----  Description:                                                        ----
----  This counter is a parametrizable clock divider. The count value is  ----
----  the generic parameter COUNT. It has a chip enable ce_i input.       ----
----  (will count only if CE is high).                                    ----
----  When it overflows, will emit a pulse on o_o.                        ----
----                                                                      ----
----  To Do:                                                              ----
----  -                                                                   ----
----                                                                      ----
----  Author:                                                             ----
----    - Philippe Carton, philippe.carton2 libertysurf.fr                ----
----    - Juan Pablo Daniel Borgna, jpdborgna gmail.com                   ----
----    - Salvador E. Tropea, salvador inti.gob.ar                        ----
----                                                                      ----
------------------------------------------------------------------------------
----                                                                      ----
---- Copyright (c) 2001-2003 Philippe Carton                              ----
---- Copyright (c) 2005 Juan Pablo Daniel Borgna                          ----
---- Copyright (c) 2005-2008 Salvador E. Tropea                           ----
---- Copyright (c) 2005-2008 Instituto Nacional de Tecnología Industrial  ----
----                                                                      ----
---- Distributed under the GPL license                                    ----
----                                                                      ----
------------------------------------------------------------------------------
----                                                                      ----
---- Design unit:      BRGen(Behaviour) (Entity and architecture)         ----
---- File name:        br_gen.vhdl                                        ----
---- Note:             None                                               ----
---- Limitations:      None known                                         ----
---- Errors:           None known                                         ----
---- Library:          zpu                                                ----
---- Dependencies:     IEEE.std_logic_1164                                ----
---- Target FPGA:      Spartan                                            ----
---- Language:         VHDL                                               ----
---- Wishbone:         No                                                 ----
---- Synthesis tools:  Xilinx Release 9.2.03i - xst J.39                  ----
---- Simulation tools: GHDL [Sokcho edition] (0.2x)                       ----
---- Text editor:      SETEdit 0.5.x                                      ----
----                                                                      ----
------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;



entity brgen is
	generic(count: integer range 0 to 65535);	-- Count revolution
	port(clk		: in  std_logic;			-- Clock
		 reset		: in  std_logic;			-- Reset input
		 ce			: in  std_logic;			-- Chip Enable
		 output		: out std_logic);			-- Output
end entity brgen;



architecture behave_brgen of brgen is

begin
	countGen:
	if (count /= 1) generate
		counter:
		process (clk, reset)
		variable cnt	: integer range 0 to count - 1;
		begin
			if (clk'event and clk = '1') then
				output <= '0';
				if (reset = '1') then
					cnt := count - 1;
				elsif (ce = '1') then
					if (cnt = 0) then
						output <= '1';
						cnt := count - 1;
					else
						cnt := cnt - 1;
					end if;
				end if;
			end if;
		end process counter;
	end generate countGen;

	countWire:
	if (count = 1) generate
		output <= '0' when (reset = '1') else ce;
	end generate countWire;
end architecture behave_brgen;
