------------------------------------------------------------------------------
----                                                                      ----
----  RS-232 simple Tx module                                             ----
----                                                                      ----
----  http://www.opencores.org/                                           ----
----                                                                      ----
----  Description:                                                        ----
----  Implements a simple 8N1 tx module for RS-232.                       ----
----                                                                      ----
----  To Do:                                                              ----
----  -                                                                   ----
----                                                                      ----
----  Author:                                                             ----
----    - Philippe Carton, philippe.carton2 libertysurf.fr                ----
----    - Juan Pablo Daniel Borgna, jpdborgna gmail.com                   ----
----    - Salvador E. Tropea, salvador inti.gob.ar                        ----
----                                                                      ----
------------------------------------------------------------------------------
----                                                                      ----
---- Copyright (c) 2001-2003 Philippe Carton                              ----
---- Copyright (c) 2005 Juan Pablo Daniel Borgna                          ----
---- Copyright (c) 2005-2008 Salvador E. Tropea                           ----
---- Copyright (c) 2005-2008 Instituto Nacional de Tecnología Industrial  ----
----                                                                      ----
---- Distributed under the GPL license                                    ----
----                                                                      ----
------------------------------------------------------------------------------
----                                                                      ----
---- Design unit:      TxUnit(Behaviour) (Entity and architecture)        ----
---- File name:        Txunit.vhdl                                        ----
---- Note:             None                                               ----
---- Limitations:      None known                                         ----
---- Errors:           None known                                         ----
---- Library:          zpu                                                ----
---- Dependencies:     IEEE.std_logic_1164                                ----
----                   zpu.UART                                           ----
---- Target FPGA:      Spartan                                            ----
---- Language:         VHDL                                               ----
---- Wishbone:         No                                                 ----
---- Synthesis tools:  Xilinx Release 9.2.03i - xst J.39                  ----
---- Simulation tools: GHDL [Sokcho edition] (0.2x)                       ----
---- Text editor:      SETEdit 0.5.x                                      ----
----                                                                      ----
------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;



entity txunit is
	port(clk		: in  std_logic;								-- Clock signal
		 reset		: in  std_logic;								-- Reset input
		 enable		: in  std_logic;								-- Enable input
		 load		: in  std_logic;								-- Load input
		 txd		: out std_logic;								-- RS-232 data output
		 busy		: out std_logic;								-- Tx Busy
		 dataIn		: in  std_logic_vector(7 downto 0));			-- Byte to transmit
end entity txunit;



architecture behave_txunit of txunit is

	signal transmitBuffer	: std_logic_vector(7 downto 0);		-- transmit buffer
	signal transmit			: std_logic_vector(7 downto 0);		-- transmit register
	signal BufferLoaded		: std_logic := '0';						-- Buffer loaded
	signal txBufferReady	: std_logic := '1';						-- Tx buffer ready

begin
	busy <= load or BufferLoaded;
	txd  <= txBufferReady;

	-- Tx process
	txProc:
	process(clk, reset)
	variable bitpos	: integer range 0 to 10;						-- Bit position in the frame
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				BufferLoaded <= '0';
				bitpos := 0;
				txBufferReady <= '1';
			else
				if (load = '1') then
					transmitBuffer <= dataIn;
					BufferLoaded <= '1';
				end if;
				if (enable = '1') then
					case bitpos is
						when 0 =>									-- idle or stop bit
							txBufferReady <= '1';
							if (BufferLoaded = '1') then			-- start transmit. next is start bit
								transmit <= transmitBuffer;
								BufferLoaded <= '0';
								bitpos := 1;
							end if;
						when 1 =>									-- Start bit
							txBufferReady <= '0';
							bitpos := 2;
						when others =>
							txBufferReady <= transmit(bitpos - 2);	-- Serialisation of transmit
							bitpos := bitpos + 1;
					end case;
					if (bitpos = 10) then							-- bit8. next is stop bit
						bitpos := 0;
					end if;
				end if;
			end if;
		end if;
	end process txProc;
end architecture behave_txunit;
