----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		font_test_gen - behave_font_test_gen 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		Font generator
--
-- Dependencies:		no
--
-- Revision:			0.02
-- Revision 0.01 - File received from Dennis Trebbels, trebbelsd@gmx.de
-- Revision 0.02 - Modified that it prints a specified text on the screen
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity font_test_gen is
   port(
      clk		: in std_logic;
		
      video_on	: in std_logic;
      
		pixel_x	: std_logic_vector (9 downto 0);
		pixel_y	: std_logic_vector (9 downto 0);
      
		rgb_text	: out std_logic_vector (2 downto 0)
   );
end font_test_gen;

architecture behave_font_test_gen of font_test_gen is
   signal rom_addr	: std_logic_vector (10 downto 0);
   signal char_addr	: std_logic_vector (6 downto 0);
   signal row_addr	: std_logic_vector (3 downto 0);
   signal bit_addr	: std_logic_vector (2 downto 0);
   signal font_word	: std_logic_vector (7 downto 0);
   signal font_bit	: std_logic;
	signal text_bit_on: std_logic;
	
begin
   -- instantiate font ROM
   font_unit: entity work.font_rom
      port map(
			clk	=> clk,
			addr	=> rom_addr,
			data	=> font_word
		);
		
   -- font ROM interface
	--> screen size: 80x30 = 2400 letters = 100101100000b
	--                           000000000000b to 100101011111b
	-- print a specific text on the screen
	process (pixel_y, pixel_x)
	begin
		if (pixel_y < 16) then
			if (pixel_x < "0000001000") then
				char_addr <= "1001010";-- J = 0x4A --pixel_y(5 downto 4) & pixel_x(7 downto 3);
			elsif (pixel_x < 16) then
				char_addr <= "1101111";-- o = 0x6F
			elsif (pixel_x < 24) then
				char_addr <= "1101000";-- h = 0x68
			elsif (pixel_x < 32) then
				char_addr <= "1100001";-- a = 0x61
			elsif (pixel_x < 40) then
				char_addr <= "1101110";-- n = 0x6E
			elsif (pixel_x < 48) then
				char_addr <= "1101110";-- n = 0x6E
			elsif (pixel_x < 56) then
				char_addr <= "1100101";-- e = 0x65
			elsif (pixel_x < 64) then
				char_addr <= "1110011";-- s = 0x73
			elsif (pixel_x < 72) then
				char_addr <= "0100000";-- ' ' = 0x20
			elsif (pixel_x < 80) then
				char_addr <= "1010011";-- S = 0x53
			elsif (pixel_x < 88) then
				char_addr <= "1110100";-- t = 0x74
			elsif (pixel_x < 96) then
				char_addr <= "1100101";-- e = 0x65
			elsif (pixel_x < 104) then
				char_addr <= "1110101";-- u = 0x75
			elsif (pixel_x < 112) then
				char_addr <= "1100100";-- d = 0x64
			elsif (pixel_x < 120) then
				char_addr <= "1101100";-- l = 0x6C
			elsif (pixel_x < 128) then
				char_addr <= "1100101";-- e = 0x65
			else
				char_addr<=(others => '0');
			end if;
		else
			char_addr<=(others => '0');
		end if;
	end process;
	
   row_addr <= pixel_y(3 downto 0);
   rom_addr <= char_addr & row_addr;
   bit_addr <= pixel_x(2 downto 0);
   font_bit <= font_word (to_integer (unsigned (not bit_addr)));
	
   -- "on" region limited to top-left corner
   text_bit_on <=
      font_bit;-- when pixel_x(9 downto 8)="00" and
               --     pixel_y(9 downto 6)="0000" else
					--'0';
		
   -- rgb multiplexing circuit
   process(video_on,font_bit,text_bit_on)
   begin
      if video_on='0' then
          rgb_text <= "000"; --blank
      else
        if text_bit_on='1' then
				-- set font color
            rgb_text <= "111"; -- white
        else
            rgb_text <= "000"; -- black
         end if;
      end if;
   end process;
	
end behave_font_test_gen;
