-- ZPU
--
-- Copyright 2004-2008 oharboe - Øyvind Harboe - oyvind.harboe@zylin.com
-- 
-- The FreeBSD license
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions
-- are met:
-- 
-- 1. Redistributions of source code must retain the above copyright
--    notice, this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above
--    copyright notice, this list of conditions and the following
--    disclaimer in the documentation and/or other materials
--    provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE ZPU PROJECT ``AS IS'' AND ANY
-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-- PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-- ZPU PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
-- INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-- OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-- HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
-- STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
-- ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- 
-- The views and conclusions contained in the software and documentation
-- are those of the authors and should not be interpreted as representing
-- official policies, either expressed or implied, of the ZPU Project.
-- 
-- 
-- Some modifications done by TU Dortmund, Computer Sience 12, ESS.


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package zpu_config is
	constant wordPower			: integer := 5;

	-- This is the msb address bit. bytes = 2 ^ (maxAddrBitIncIO + 1)
	constant maxAddrBitIncIO	: integer := 31;						-- old value: 27

	constant byteBits			: integer := wordPower - 3;				-- # of bits in a word that addresses bytes
	constant maxAddrBit			: integer := maxAddrBitIncIO - 1;
	constant ioBit				: integer := maxAddrBit + 1;
	constant wordSize			: integer := 2 ** wordPower;
	constant wordBytes			: integer := wordSize / 8;
	constant minAddrBit			: integer := byteBits;
	-- configurable internal stack size. Probably going to be 16 after toolchain is done
	constant stack_bits			: integer := 5;
	constant stack_size			: integer := 2 ** stack_bits;


	--	| RAM size     	| maxAddrBitBRAM	| Block-RAMs	| Memory size  	|
	--	|-----------------|-----------------|--------------|-----------------|
	--	|		32 KB		   |		14	   		|		16  		|		S2	(-)		|
	--	|		16 KB	  	   |		13			   |		 8		   |		S4	(-)		|
	--	|		 8 KB		   |		12			   |		 4		   |		S9	(1)		|
	--	|		 4 KB		   |		11			   |		 2		   |		S18	(2)	|
	--	|		 2 KB		   |		10			   |		 1		   |		S36	(4)	|

	constant maxAddrBitBRAM1		: integer := 13;


	-- start byte address of stack. point to top of RAM - 2*words
	constant spStart1			: std_logic_vector(maxAddrBitIncIO downto 0) := std_logic_vector(to_unsigned((2 ** (maxAddrBitBRAM1 + 1)) - 8, maxAddrBitIncIO + 1));
end zpu_config;
