library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.softcore_config.all;

package softcore_pkg is

	-- Wishbone implementation
	type wb_master_in is record
		data		: std_logic_vector(31 downto 0);
		ack		: std_logic;
	end record;

	type wb_master_out is record
		addr		: std_logic_vector(31 downto 0);
		data		: std_logic_vector(31 downto 0);
		sel		: std_logic_vector(3 downto 0);
		we			: std_logic;
		stb		: std_logic;
		cyc		: std_logic;
	end record;
	
	

	-- Peripheral devices and core implementation
	component wb_core is
		port(clk					: in  std_logic;
			 reset				: in  std_logic;
			 master_out			: out wb_master_out;
			 master_in			: in  wb_master_in;
			 interrupt			: in  std_logic);
	end component;

	component interrupt is
		generic(irqLines		: integer := 0);
		port(clk					: in  std_logic;
			 reset				: in  std_logic;
			 irq_core			: out std_logic;
			 irq					: in  std_logic_vector(irqLines - 1 downto 0);
			 slave_in			: in  wb_master_out;
			 slave_out			: out wb_master_in);
	end component;

	component uart is
		generic(brdivisor	: integer := 0);
		port(clk				: in  std_logic;
			 reset			: in  std_logic;
			 interrupt		: out std_logic;
			 uart_rx			: in  std_logic;
			 uart_tx			: out std_logic;
			 slave_in		: in  wb_master_out;
			 slave_out		: out wb_master_in);
	end component;
	
	component ps2 is
		port(clk				: in  std_logic;
			 reset			: in  std_logic;
			 interrupt		: out std_logic;															
			 slave_in		: in  wb_master_out;
			 slave_out		: out wb_master_in;
			 ps2clk			: inout std_logic;
			 DataIn			: inout std_logic);
	end component;
	
	component intIO is
		port(clk				: in  std_logic;
			 reset			: in  std_logic;
			 interrupt		: out std_logic;
			 slave_in		: in  wb_master_out;
			 slave_out		: out wb_master_in;
			 LEDs				: out std_logic_vector(7 downto 0);
			 switches		: in std_logic_vector(7 downto 0);
			 sseg_on			: out std_logic_vector(3 downto 0);
			 sseg				: out std_logic_vector(7 downto 0);
			 buttons			: in std_logic_vector(3 downto 0)
			 );
	end component;
	
	component wb_sram is
		port(
			clk, reset		: in std_logic;
			interrupt		: out std_logic;
			slave_in			: in  wb_master_out;
			slave_out		: out wb_master_in;
			ad					: out std_logic_vector(17 downto 0);
			we_n				: out std_logic;
			oe_n				: out std_logic;
			dio_a				: inout std_logic_vector(15 downto 0);
			ce_a_n			: out std_logic;
			ub_a_n			: out std_logic;
			lb_a_n			: out std_logic
		);
	end component;
	
	component wb_vga is
		port
		(
			clk, reset		: in std_logic;
			interrupt		: out std_logic;
			slave_in			: in  wb_master_out;
			slave_out		: out wb_master_in;
			hsync, vsync	: out std_logic;
			rgb				: out std_logic_vector(2 downto 0)
		);
	end component;

end softcore_pkg;
