library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.mpsoc_pkg.all;



entity interrupt is
	generic(irqLines	: integer := 0);
	port(clk			: in  std_logic;
		 reset			: in  std_logic;
		 irq_core		: out std_logic;
		 irq			: in  std_logic_vector(irqLines - 1 downto 0);
		 slave_in		: in  wb_master_out;
		 slave_out		: out wb_master_in);
end entity interrupt;



architecture behave_interrupt of interrupt is

signal globalEnable			: std_logic;
signal interruptEnable		: std_logic_vector(irqLines - 1 downto 0);

begin

	irq_core <= '1' when (((irq and interruptEnable) /= std_logic_vector(to_unsigned(0, irqLines))) and (globalEnable = '1')) else '0';


	slave_out.ack <= slave_in.cyc;
	slave_out.data <= std_logic_vector(to_unsigned(0, (31 - irqLines + 1))) & (irq and interruptEnable) when (slave_in.cyc = '1' and slave_in.addr(3 downto 2) = "01") else		-- Lesen der IRQs
					  std_logic_vector(to_unsigned(0, (31 - irqLines + 1))) & interruptEnable when (slave_in.cyc = '1' and slave_in.addr(3 downto 2) = "10") else				-- Lesen der Interrupts
					  "0000000000000000000000000000000" & globalEnable when (slave_in.cyc = '1' and slave_in.addr(3 downto 2) = "11") else										-- Lesen des globalen Interrupt-Flags
					  (others => '0');


	interrupt_process:
	process(clk, reset)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				globalEnable <= '0';
				interruptEnable <= (others => '0');
			else


				if (slave_in.cyc = '1' and slave_in.we = '1') then
					-- Setzen/Loeschen der Interrupts
					if (slave_in.addr(3 downto 2) = "10") then
						interruptEnable <= slave_in.data(irqLines - 1 downto 0);

					-- Setzen/Loeschen des globalen Interrupt-Flags
					elsif (slave_in.addr(3 downto 2) = "11") then
						globalEnable <= slave_in.data(0);
					end if;
				end if;
			end if;
		end if;
	end process;
end architecture behave_interrupt;
