library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.mpsoc_config.all;
use work.zpu_config.all;

package mpsoc_pkg is

	-- Wishbone Implementierung
	type wb_master_in is record
		data	: std_logic_vector(31 downto 0);
		ack		: std_logic;
	end record;

	type wb_master_out is record
		addr	: std_logic_vector(31 downto 0);
		data	: std_logic_vector(31 downto 0);
		sel		: std_logic_vector(3 downto 0);
		we		: std_logic;
		stb		: std_logic;
		cyc		: std_logic;
	end record;


	component soc_top is
		generic(coreID			: std_logic_vector(cores - 1 downto 0) := (others => '0');
				waitTime		: integer := 1);
		port(clk				: in  std_logic;
			 reset				: in  std_logic;
			 uart_rx1			: in  std_logic;
			 uart_tx1			: out std_logic);
	end component;

	component wb_core is
		port(clk				: in  std_logic;
			 reset				: in  std_logic;
			 master_out			: out wb_master_out;
			 master_in			: in  wb_master_in;
			 interrupt			: in  std_logic);
	end component;

	component interrupt is
		generic(irqLines		: integer := 0);
		port(clk				: in  std_logic;
			 reset				: in  std_logic;
			 irq_core			: out std_logic;
			 irq				: in  std_logic_vector(irqLines - 1 downto 0);
			 slave_in			: in  wb_master_out;
			 slave_out			: out wb_master_in);
	end component;

	component uart is
		generic(brdivisor	: integer := 0);
		port(clk			: in  std_logic;
			 reset			: in  std_logic;
			 interrupt		: out std_logic;
			 uart_rx		: in  std_logic;
			 uart_tx		: out std_logic;
			 slave_in		: in  wb_master_out;
			 slave_out		: out wb_master_in);
	end component;

end mpsoc_pkg;
