library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library work;
use work.mpsoc_config.all;
use work.mpsoc_pkg.all;
use work.zpu_config.all;



entity soc_top is
	generic(coreID			: std_logic_vector(cores - 1 downto 0) := (others => '0');
			waitTime		: integer := 1);
	port(clk				: in  std_logic;
		 reset				: in  std_logic;
		 uart_rx1			: in  std_logic;
		 uart_tx1			: out std_logic);
end soc_top;



architecture behave_soc_top1 of soc_top is

signal master_out		: wb_master_out;
signal master_in		: wb_master_in;
signal dummy_out		: wb_master_in;

signal irq_core			: std_logic;
signal irq_out			: wb_master_in;
signal irq_in			: wb_master_out;
signal irq				: std_logic_vector(1 - 1 downto 0);

signal uart1_out		: wb_master_in;
signal uart1_in			: wb_master_out;
signal irq_uart1		: std_logic;

for core : wb_core use entity work.wb_core(behave_zpu1);


begin

	core: wb_core
		port map(clk => clk,
				 reset => reset,
				 master_out => master_out,
				 master_in  => master_in,
				 interrupt => irq_core);


	interrupt_inst: interrupt
		generic map(irqLines => 1)
		port map(clk => clk,
				 reset => reset,
				 irq_core => irq_core,
				 irq => irq,
				 slave_in => irq_in,
				 slave_out => irq_out);
				
	irq(0) <= irq_uart1;

	irq_in.addr <= master_out.addr;
	irq_in.data <= master_out.data;
	irq_in.sel  <= master_out.sel;
	irq_in.we   <= master_out.we;
	irq_in.stb  <= master_out.stb;
	irq_in.cyc  <= master_out.cyc when (master_out.addr(31 downto 11) = "1000" & '0' & std_logic_vector(to_unsigned(0, (26 - 11 + 1)))) else '0';


	uart_inst_1: uart
		generic map(brdivisor => brdivisor1)
		port map(clk => clk,
				 reset => reset,
				 interrupt => irq_uart1,
				 uart_rx => uart_rx1,
				 uart_tx => uart_tx1,
				 slave_in => uart1_in,
				 slave_out => uart1_out);

	uart1_in.addr <= master_out.addr;
	uart1_in.data <= master_out.data;
	uart1_in.sel  <= master_out.sel;
	uart1_in.we   <= master_out.we;
	uart1_in.stb  <= master_out.stb;
	uart1_in.cyc  <= master_out.cyc when (master_out.addr(31 downto 11) = "1000" & '0' & "0000001000000010") else '0';


	-- Dummy liegt am Master an, wenn Adresse zu keiner Komponente passt
	dummy_out.ack <= '0';
	dummy_out.data <= (others => '0');

	master_in <=
		irq_out when (master_out.addr(31 downto 11) = "1000" & '0' & std_logic_vector(to_unsigned(0, (26 - 11 + 1)))) else
		uart1_out when (master_out.addr(31 downto 11) = "1000" & '0' & "0000001000000010") else
		dummy_out;
end behave_soc_top1;
