library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.mpsoc_config.all;
use work.mpsoc_pkg.all;
use work.zpu_config.all;
use work.zpu_pkg.all;



entity wb_core is
	generic(coreID			: std_logic_vector(cores - 1 downto 0) := (others => '0'));
	port(clk				: in  std_logic;
		 reset				: in  std_logic;
		 master_out			: out wb_master_out;
		 master_in			: in  wb_master_in;
		 interrupt			: in  std_logic);
end wb_core;


architecture behave_zpu1 of wb_core is

	-- CPU
	signal mem_busy        : std_logic;
	signal mem_read        : std_logic_vector(wordSize - 1 downto 0);
	signal mem_write       : std_logic_vector(wordSize - 1 downto 0);
	signal mem_addr        : std_logic_vector(maxAddrBitIncIO downto 0);
	signal mem_writeEnable : std_logic;
	signal mem_readEnable  : std_logic;
	signal mem_writeMask   : std_logic_vector(wordBytes - 1 downto 0);

	type State_Type is(Idle, AckWait);
	signal state : State_Type;

	for zpu_core_1 : zpu_core use entity work.zpu_core(behave_zpu_core1);


begin

	-- CPU Implementierung
	zpu_core_1 : zpu_core
		port map (
			clk                 => clk,
			reset               => reset,
			in_mem_ack          => mem_busy,
			mem_read            => mem_read,
			mem_write           => mem_write,
			out_mem_addr        => mem_addr,
			out_mem_writeEnable => mem_writeEnable,
			out_mem_readEnable  => mem_readEnable,
			mem_writeMask       => mem_writeMask,
			interrupt           => interrupt);

	-- Wishbone Implementierung
	mem_busy <= master_in.ack;
	mem_read <= master_in.data;

	process(clk, reset)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				master_out.stb <= '0';
				master_out.cyc <= '0';
				master_out.we <= '0';
				state <= Idle;
			else

				-- Zustandsmaschine
				case state is
					when Idle =>
						if (mem_addr(31 downto 28) = "1000" and (mem_writeEnable = '1' or mem_readEnable = '1')) then
							master_out.addr <= mem_addr;
							master_out.data <= mem_write;
							master_out.sel <= mem_writeMask;
							master_out.stb <= '1';
							master_out.cyc <= '1';
							master_out.we <= mem_writeEnable;
							state <= AckWait;
						end if;

					when AckWait =>
						if (mem_busy = '1') then
							master_out.stb <= '0';
							master_out.cyc <= '0';
							master_out.we <= '0';
							state <= Idle;
						end if;
				end case;
			end if;
		end if;
	end process;
end behave_zpu1;
